/*																							
			Permission to use		
																							
 

  Sony Computer Entertainment supplies this lws2rsd software free 	  
  of charge to licensees along with the source code files.		

  This lws2rsd software is experimental and no support of any kind is 	  
  available.							  

  The licensee is hereby granted permission to use, copy, modify, and	  
  distribute this lws2rsd software and documentation for any purpose. 	  
  Sony Computer Entertainment shall bear no responsibility for any 	  
  damage or loss sustained either directly or indirectly as a result of	 
  using this software. 						

  Copyright 1997 by Sony Computer Entertainment Inc. All Rights Reserved.

*/
																						

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "myTypes.h"
#include "io.h"
#include "vecmat.h"
#include "LW.h"
#include "cap.h"


const Matrix PS_SYSTEM_MATRIX =
{ 
	(Scalar)0.0,	(Scalar)1.0,	(Scalar)0.0,	(Scalar)0.0,
	(Scalar)0.0,	(Scalar)0.0,	(Scalar)-1.0,	(Scalar)0.0,
	(Scalar)-1.0,	(Scalar)0.0,	(Scalar)0.0,	(Scalar)0.0,
	(Scalar)0.0,	(Scalar)0.0,	(Scalar)0.0,	(Scalar)1.0
};

static int				FLTcode;
static CAPValidCode		validCode;
static RotationOrder	rotationOrder;
static AngleUnit		angleUnit;

static Matrix	SRC_SYSTEM_MATRIX;
static Matrix  	tSRC_SYSTEM_MATRIX;
static Matrix	SRC2PS_SYSTEM_MATRIX;
static Matrix  	PS2SRC_SYSTEM_MATRIX;



/******************************************************************************/
 Matrix SetupSystemMatrix( int FLTcode )
/******************************************************************************/
{
	Matrix	systemMatrix = { (Scalar)0.0, (Scalar)0.0, (Scalar)0.0, (Scalar)0.0,
							 (Scalar)0.0, (Scalar)0.0, (Scalar)0.0, (Scalar)0.0,
							 (Scalar)0.0, (Scalar)0.0, (Scalar)0.0, (Scalar)0.0,
							 (Scalar)0.0, (Scalar)0.0, (Scalar)0.0, (Scalar)1.0	 };

//  ( XXX & 0111 = 4, 2, 1 ) >> 1 ==> 2, 1, 0 axis index
//
//		FLTcode 1010 0001 1100
//	and	0x888	1000 1000 1000
//				 -1	  +1   -1				  sign
//
//		FLTcode 1010 0001 1100
//	and	0x777	0111 0111 0111
//				0010 0001 0100                0x700 >> 9, 0x70 >> 5, 0x7 >> 1
//
//				  F	   L	T
//			X	  0	   1	0
//			Y    -1    0	0
//			Z	  0	   0	1

 	systemMatrix.M[ ( FLTcode & 0x700 ) >> 9 ][0] = ( FLTcode & 0x800 ) ? (Scalar)-1.0 : (Scalar)1.0;
 	systemMatrix.M[ ( FLTcode & 0x70  ) >> 5 ][1] = ( FLTcode & 0x80  ) ? (Scalar)-1.0 : (Scalar)1.0;
 	systemMatrix.M[ ( FLTcode & 0x7   ) >> 1 ][2] = ( FLTcode & 0x8   ) ? (Scalar)-1.0 : (Scalar)1.0;

	return systemMatrix;
}


/******************************************************************************/
 int SetFLTcode( char *lineBuffer )
/******************************************************************************/
{
	FLTcode = 0;

	Ucase( lineBuffer );

	if ( lineBuffer[0] == '-' )
		FLTcode = 0x800;

	switch ( lineBuffer[1] ) {
		case 'X' : FLTcode |= 0x100; break;
		case 'Y' : FLTcode |= 0x200; break;
		case 'Z' : FLTcode |= 0x400; break;
		default  : return NG;
	}
		
	if ( lineBuffer[2] == '-' )
	  FLTcode |= 0x80;

	switch ( lineBuffer[3] ) {
		case 'X' :						if ( FLTcode & 0x100 )	return NG;
					FLTcode |= 0x10;	break;
		case 'Y' :						if ( FLTcode & 0x200 )	return NG;
					FLTcode |= 0x20;	break;
		case 'Z' :						if ( FLTcode & 0x400 )	return NG;
					FLTcode |= 0x40;	break;
		default  :						return -1;
	}
		
	if ( lineBuffer[4] == '-' )
		FLTcode |= 0x8;

	switch ( lineBuffer[5] ) {
		case 'X' :						if ( FLTcode & 0x110 )	return NG;			 
					FLTcode |= 0x1;		break;
		case 'Y' :                      if ( FLTcode & 0x220 )	return NG;
					FLTcode |= 0x2;		break;
		case 'Z' :						if ( FLTcode & 0x440 )	return NG;
					FLTcode |= 0x4;		break;
		default  :						return -1;
	}

	SRC_SYSTEM_MATRIX	 = SetupSystemMatrix( FLTcode );
	tSRC_SYSTEM_MATRIX	 = TransposeMatrix( &SRC_SYSTEM_MATRIX );
	SRC2PS_SYSTEM_MATRIX = MatrixMult( (MatrixPtr)&PS_SYSTEM_MATRIX, &tSRC_SYSTEM_MATRIX );
	PS2SRC_SYSTEM_MATRIX = TransposeMatrix( &SRC2PS_SYSTEM_MATRIX );

	return FLTcode;
}


/******************************************************************************/
 CAPValidCode SetValidCode( char *lineBuffer  )
/******************************************************************************/
{
	validCode = atoi( lineBuffer );
	return validCode;
}


/******************************************************************************/
 RotationOrder SetRotationOrder( char *lineBuffer )
/******************************************************************************/
{
	Ucase( lineBuffer );

	if ( strncmp( lineBuffer,  "XYZ", (size_t)3 ) == 0 )
		rotationOrder = ROTATION_ORDER_XYZ;			  
	else if ( strncmp( lineBuffer,  "XZY", (size_t)3 ) == 0 )
		rotationOrder = ROTATION_ORDER_XZY;			  
	else if ( strncmp( lineBuffer,  "YXZ", (size_t)3 ) == 0 )
		rotationOrder = ROTATION_ORDER_YXZ;			  
	else if ( strncmp( lineBuffer,  "YZX", (size_t)3 ) == 0 )
		rotationOrder = ROTATION_ORDER_YZX;		
	else if ( strncmp( lineBuffer,  "ZXY", (size_t)3 ) == 0 )
		rotationOrder = ROTATION_ORDER_ZXY;		
	else if ( strncmp( lineBuffer,  "ZYX", (size_t)3 ) == 0 )
		rotationOrder = ROTATION_ORDER_ZYX;

	return rotationOrder;
}


/******************************************************************************/
 AngleUnit SetAngleUnit( char *lineBuffer  )
/******************************************************************************/
{
	angleUnit = atoi( lineBuffer );
	return angleUnit;
}


/******************************************************************************/
 Matrix SetRotationMatrix2( ParamPtr angle, RotationOrder order )
/******************************************************************************/
{
	Matrix		rotX, rotY, rotZ, rot;

	rotX = SetXRotationMatrix( angle->x );
	rotY = SetYRotationMatrix( angle->y );
	rotZ = SetZRotationMatrix( angle->z );

	switch ( order ) {
		case ROTATION_ORDER_XYZ :
			rot = MatrixMult( &rotY, &rotX );
			rot = MatrixMult( &rotZ, &rot  );
			break;
		case ROTATION_ORDER_XZY :
			rot = MatrixMult( &rotZ, &rotX );
			rot = MatrixMult( &rotY, &rot  );
			break;
		case ROTATION_ORDER_YXZ :
			rot = MatrixMult( &rotX, &rotY );
			rot = MatrixMult( &rotZ, &rot  );
			break;
		case ROTATION_ORDER_YZX :
			rot = MatrixMult( &rotZ, &rotY );
			rot = MatrixMult( &rotX, &rot  );
			break;
		case ROTATION_ORDER_ZXY :
			rot = MatrixMult( &rotX, &rotZ );
			rot = MatrixMult( &rotY, &rot  );
			break;
		case ROTATION_ORDER_ZYX :
			rot = MatrixMult( &rotY, &rotZ );
			rot = MatrixMult( &rotX, &rot  );
			break;
	}

	return rot;
}


/******************************************************************************/
 void Transform( Scalar scaleLw2Ps, ParamPtr s, ParamPtr r, ParamPtr t, ParamPtr s2, ParamPtr r2, ParamPtr t2 )
/******************************************************************************/
 {
	Matrix	S, R, T, mat;
	Param	t1, r1;

	// LW Left Hand Coordinate System Special
	t1.x = t->x * scaleLw2Ps;
	t1.y = t->y * -scaleLw2Ps;
	t1.z = t->z * scaleLw2Ps;

	r1.x = r->x * -1;
	r1.y = r->y * 1;
	r1.z = r->z * -1;

				
	if ( ( validCode & PARAM_R ) && ( angleUnit == ANGLE_DEGREE ) )
		r1 = DEGtoRAD( &r1 );

	S = SetScalingMatrix( s );
	R = SetRotationMatrix2( &r1, rotationOrder );
	T = SetTranslationMatrix( &t1 );
	ComposeTRSfromT_R_S( &mat, &T, &R, &S );
	mat = MatrixMult( &SRC2PS_SYSTEM_MATRIX, &mat );
	mat = MatrixMult( &mat, &PS2SRC_SYSTEM_MATRIX ); 
	DecomposeTRS2t_r_s( &mat, t2, r2, s2 );
}



