/************************************************************

VH2SPU.EXE v 1.1 (author ROB VAWTER [email: devtech_support@playstation.sony.com])
copyright 1997 Sony Computer Entertainment America
	ALL RIGHTS RESERVED

HISTORY
1.1 11/18/97 Added comment to #define KEYON_CENTER_NOTE 60 (line 63) for clarity
1.0 11/13/97 Original

TRANSFORMS VABHEADER (.VH) FILE INTO 2 FILES:

.SPU FILE	contains keyon data for libspu
.OFF FILE	contains waveform data start address offsets

COMMAND LINE vh2spu vabheaderfilename (don't input the extension ".vh")

FILES INCLUDED: 
vh2spu.exe 	data converter
vh2spu.c 	source code for data converter
vh2spu.h    header file for data converter
n2p.obj		compiled source code for SsPitchFromNote(), 
				libsnd function used by vh2spu.c

This utility is designed to be used with the code changes
suggested in snd2spu.c

	THIS IS UNPUBLISHED PROPREITRARY SOURCE CODE OF SONY COMPUTER 
	ENTERTAINMENT AMERICA
	This code is sample code. Sony Computer Entertainment America and 
	the author(s) accept no responsibility for any problems which may 
	arise from using this code. This file may be used and/or modified 
	provided the disclaimer mentioned above is maintained, and the 
	appropriate Non Disclosure Agreement with Sony Computer Entertainment 
	America and the party or parties using/modifying the code exist. 
	This file may not be distributed to any party/parties which do not
	have the appropriate Non Disclosure Agreement with Sony Computer Entertainment America


**************************************************************/
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "vh2spu.h"

#define PRIORITY_USED
/* Will your voice allocator need priority levels of sounds? Only if more than 24
  sfx can play at the same time	*/

#define MODE_USED
/* Is reverb used at all OR will reverb affect some sfx and not others. If yes,
you need MODE_USED */

#define PAN_USED
/* Pan is ONLY useful for sounds which will always be keyed on in the same spatial
   location regardless of camera angle, player position, sound emitting object
   position etc. and ONLY if the sound has a spatial location other than center.
   So, it would ONLY be useful for an ambient left sound or for a stereo effect
   ambient sound.

   For all other cases, sound location calculation will be performed to change the
   base volume level set up in the vol member of the spu_data_struct structure.

   Note that keying a sound in in libspu with the following pan value will NOT
   exactly emulate libsnd in all cases. If the program pan is set to other than
   center (64) and the tone pan is set to other than center, the result will only
   reflect the tone pan. Why? Well, pan in libsnd is used to reduce the volume
   value on the side which is panned away from - example: if tone pan = 23,
   the panning effect will setup volumes as follows:
   rvol = vol * 23 / 64;
   lvol = vol;

   If the program pan is 90, the left volume will now be reduced
   lvol = lvol * (127 - 90) / 64;
   rvol = rvol; */

// #define NO_STANDARD_PITCH
/* reverse of below is true - musician provides key on notes and shifts */
#ifndef NO_STANDARD_PITCH
#define KEYON_CENTER_NOTE 60 /* this value is arbitrary in my program - use 
the master value from your program */
#define KEYON_NOTE_SHIFT 0	 
/* Only if all sfx are keyed on at the same note and fine. Range 0-127. KEYON_NOTE_SHIFT
is a percentage of one note adjustment.
If these 2 values vary from sfx to sfx, you should make an array of the values given
to you by your musician and plug this array into the SsPitchFromNote() function. */
#endif

#ifdef NO_STANDARD_PITCH
typedef struct {
unsigned char note;
unsigned char shift;
} sfx_keyon_data

#define NUM_SFX 3 //However big the damn list the musician gave you is

sfx_keyon_data key_on[NUM_SFX];
#endif


typedef struct {
unsigned char tones;
unsigned char vol;
#ifdef PRIORITY_USED
unsigned char prior;
#endif
} Prog_struct;


/* Padding these out to longs wasn't much fun, 
please feel free to do it differently */
typedef struct {
#ifdef PRIORITY_USED
unsigned char 	prior;
#else	//PRIORITY_USED
#define PRI_NOT_USED
#endif  //PRIORITY_USED

#ifdef MODE_USED
unsigned char 	mode;
#ifdef PRI_NOT_USED
#define PAD_ONE	  // only 1 of mode and pri used - 1 pad on this long
#else  // PRI_NOT_USED
#define PAD_ZERO  // both mode and pri used - no pad on this long
#endif // PRI_NOT_USED
#else //if not MODE_USED
#ifdef PRI_NOT_USED
#define PAD_CARRYOVER  // neither mode nor pri used - carry padding to PAN
#else //PRI_NOT_USED
#define PAD_ONE	  // only 1 of mode and pri used - 1 pad on this long
#endif //PRI_NOT_USED
#endif //MODE_USED


#ifdef PAD_ONE
unsigned char	pad1;
#endif //PAD_ONE

short		  	vol;


#ifdef PAN_USED
unsigned char 	pan;
#if defined(PAD_ZERO) || defined(PAD_ONE)
unsigned char	pad2;
unsigned short	pad3;
#endif // PAD_ZERO || PAD_ONE
#ifdef PAD_CARRYOVER
unsigned char	pad4;
#endif //PAD_CARRY
#else // not PAN_USED
#ifdef PAD_CARRYOVER
unsigned short	pad5;
#endif //PAD_CARRY
#endif //PAN_USED

unsigned short	pitch;
unsigned short	adsr1;

unsigned short	adsr2;
short			vagnum;

} spu_data_format;

spu_data_format vag_info; // new sfx data info

/* set up necessary program info for all 128 usable programs */
Prog_struct prog[128];

int main(int argc, char *argv[])
{
    FILE *sourcefile, *destfile1, *destfile2; // 3 files - input and 2 output
    char filename[256], srcfilename[256], dest1filename[256], dest2filename[256]; 
    										 // ditto
    unsigned char *data; // VH's data destination in memory 
    unsigned long vab_check; // used to verify that the src is in fact a VAB header
    unsigned char *vh_ptr; // pointer used in hop around data
    unsigned short tot_vags; // number of vags in VH
    unsigned char vh_vol; // master volume of VAB
    unsigned char i,j; 	  // work
    unsigned char pg_cnt, tn_cnt;  // track number of programs in VH 
    							   //	and number of tones per program
    VagAtr *tn_info; // pointer to each tone info of VH
    ProgAtr *pg_info; // pointer to each program info of VH
    VabHdr *vh_info; // pointer to vabhdr info of VH
	unsigned long vag_offset, offset; // running VAG offset, individual vag offset
    short vagsize; // how big is each vag?

    // You typed this:
    for (i = 0; i< argc; i++) printf("%s ", argv[i]);
    printf("\n");

    // vh2spu expects this:
    if (argc != 2)
    {
        printf("Usage: VH2SPU filename\n");
        printf("    filename = a valid *.vh file\n");
        printf("    DO NOT ENTER THE EXTENSION .VH\n");
        return -1;
    }

    /* create filename string */
    strcpy(filename, argv[1]);

	/* create srcfilename and add .vh to it */
    strcpy(srcfilename, filename);
    strcat(srcfilename, ".vh");

	/* open source file - VH data */
    if (sourcefile=fopen(srcfilename,"rb"))
	{
		/* malloc needed area for data from VH */
        data = (unsigned char *)malloc(sizeof(VabHdr)+128*sizeof(ProgAtr)+16*128*sizeof(VagAtr)+510);
        if ((int)data < 0)
        {
            printf("malloc failed\n");
            return -1;
        }
		/* need to change this to malloc some area of memory and
		   read in the entire file...hmmm maybe read in the first 4 bytes first */
		/* read first 32 bytes to verify VH */
        i = fread(data,sizeof(VabHdr),1,sourcefile);
	}
	else
	{
        printf("Error reading the source file\n");
        return -1;
	}

	/* set ptr to start of vab header */
	vh_ptr = (unsigned char*) data;

	/* set up vabheader info */
	vh_info = (VabHdr*) vh_ptr;

	/* verify VH file */
	vab_check = vh_info->form;
	if (vab_check>>8 !=0x564142)
	{
		/* change this to write to command line */
        printf("The source file is not a VAB\n");
        return -1;
	}

	/* verify VH version - very, very old VABs not converted. 
	How old? Before October 1995; 
	that's when I started, beyond that I don't know*/
	if (vh_info->ver<5)
	{
		/* change this to write to command line */
        printf("This version VAB is too old to convert\n");
        return -1;
	}

	/* create and open destination file1 - change extension to .spu */
    strcpy(dest1filename, filename);
    strcat(dest1filename, ".spu");
    if(!(destfile1 = fopen(dest1filename,"wb")))     // note to self wb = write, binary
    {
        printf("Could not open destination file\n");
        return -1;
	}

    /* create and open destination file2 - change extension to .off */
    strcpy(dest2filename, filename);
    strcat(dest2filename, ".off");
    if(!(destfile2=fopen(dest2filename,"wb")))     // note to self wb = write, binary
    {
        printf("Could not open destination file\n");
        return -1;
	}

	printf("Processing: %s -> %s, %s\n",srcfilename, dest1filename, dest2filename); 

	/* increment data start for second transfer */
	data += sizeof(VabHdr);

	/* transfer rest of file */
    fread(data,sizeof(sourcefile)-sizeof(VabHdr),1,sourcefile);

	/* track number of vags */
	tot_vags = vh_info->vs;

	/* check master volume */
	vh_vol = vh_info->mvol;

	/* move ptr to start of programs */
	vh_ptr += sizeof(VabHdr);

	/* init program count */
	pg_cnt = 0;

	/* search thru all possible programs (blank programs may exist) */
    for (i = 0; i<128; i++)
	{
		/* setup ProgAtr */
		pg_info = (ProgAtr*)vh_ptr;

		/* any tones in this program?? */
		if (pg_info->tones>0)
		{
			/* store number of tones per program */
			prog[pg_cnt].tones = pg_info->tones;

			/* store program volume */
		 	prog[pg_cnt].vol = pg_info->mvol;

			/* increment number of programs */
			pg_cnt++;
		}

		/* advance vh_ptr to start of next program */
		vh_ptr += sizeof(ProgAtr);
	}

	/* vh_ptr should now be pointing to the start of tone area */
    for (i = 0; i<pg_cnt; i++)
	{

		tn_cnt = 0;
		/* It is accidentally possible to make gaps in the tone bank
		   However, libsnd keyon won't work properly with this anyway, 
		   so I didn't account for it; pray your musician didn't do it */
        for (j = 0; j<prog[i].tones; j++)
		{

			/* set up tone info */
			tn_info = (VagAtr*)vh_ptr;

#ifdef PRIORITY_USED
			/* store sfx priority */
			vag_info.prior = tn_info->prior;
#endif

#ifdef MODE_USED
			/* store reverb (could also be portamento or vibrato, but those
			are not currently supported, and likely won't ever be) */
			vag_info.mode = tn_info->mode;
#endif
			/* calculate volume of sfx relative to all other sfx possible volumes
			take into account tone volume, program volume and master vab volume as
			percents of 127 */
			vag_info.vol = (((0x3fff*tn_info->vol/127) * prog[i].vol/127) * vh_vol)/127;

#ifdef PAN_USED
			/* store partial sfx pan - see note at top */
			vag_info.pan  = tn_info->pan;

#endif
#ifndef NO_STANDARD_PITCH

			/* convert to short int pitch value using standard note */
			vag_info.pitch = SsPitchFromNote(KEYON_CENTER_NOTE, KEYON_NOTE_SHIFT,
			   tn_info->center, tn_info->shift);
#else
			/* convert to short int pitch value using musician list array */
			vag_info.pitch = SsPitchFromNote(keyon[tn_cnt].note, keyon[tn_cnt].shift, tn_info->center, tn_info->shift);
#endif

	/* store envelope info */
			vag_info.adsr1 = tn_info->adsr1;
			vag_info.adsr2 = tn_info->adsr2;

			/* Subtract 1 to shift the vags from 1-254 to 0-253. That way
			   the vag offest array can be checked from 0-253 */
			vag_info.vagnum	= tn_info->vag-1;


			/* write one sfx data worth of info to the new file "filename.spu" */
            fwrite(&vag_info,sizeof(vag_info),1,destfile1);

			/* advance to next tone */
			vh_ptr += sizeof(VagAtr);
        }

		/* advance through final unused tones for this program
		that is, go to start of tones for next program or start of vag offset data */
		vh_ptr += sizeof(VagAtr)*(16-j);
	}
	/* init vag_offset */
	vag_offset = 0;

	/* extract vag offset data from VH file */
	for (i = 0; i<tot_vags; i++)
	{
		  /* extract first VAG offset 2 bytes at a time*/
          vagsize = *(short *)vh_ptr;

		  /* convert to unsigned long */
          offset = ((unsigned long)vagsize) &0xFFFF;

		  /* left shift 3 bytes to get actual size */
          offset <<= 3;

		  /* add to running_offset */
		  vag_offset +=offset;

    	  /* write one vagsize to the new file "filename.spu" */
          fwrite(&vag_offset,sizeof(vag_offset),1,destfile2);

		  /* increment to start of next vag offset */
		  vh_ptr +=2;
	}
	/* close all 3 files */
	fclose(sourcefile);
	fclose(destfile1);
	fclose(destfile2);

    return 0;
}
/* end of file */






