/* $PSLibId: Runtime Library Version 3.5 Equivalent$ */
#ifndef _LIBGPU_H_
#define _LIBGPU_H_
/*
 *  (C) Copyright 1994-1996 Sony Computer Entertainment Inc. All Rights Reserved.
 *
 * 		libgpu.h: Graphic Primitive Structures Database	for MS Visual C++
 *
 * Primitive list:
 *
 *      Name     |Size*1|Shade  |Vertex |Texture| Function
 *      ---------+------+-------+-------+-------+------------------------
 *      POLY_F3  | 6	|Flat   |   3   |OFF    | Flat Triangle
 *      POLY_FT3 | 9	|Flat   |   3   |ON     | Flat Textured Triangle
 *      POLY_G3  | 8	|Gouraud|   3   |OFF    | Gouraud Triangle
 *      POLY_GT3 |11	|Gouraud|   3   |ON     | Gouraud Textured Triangle
 *      POLY_F4  | 7	|Flat   |   4   |OFF    | Flat Quadrangle
 *      POLY_FT4 |11	|Flat   |   4   |ON     | Flat Textured Quadrangle
 *      POLY_G4  |10	|Gouraud|   4   |OFF    | Gouraud Quadrangle
 *      POLY_GT4 |14	|Gouraud|   4   |ON     | Gouraud Textured Quadrangle
 *      ---------+------+-------+-------+-------+------------------------
 *      LINE_F2  | 5	|Flat   |   2   | -	| unconnected Flat Line 
 *      LINE_G2  | 6	|Gouraud|   2   | -	| unconnected Gouraud Line 
 *      LINE_F3  | 7	|Flat	|   3	| -	| 3-connected Flat Line
 *      LINE_G3  | 9	|Gouraud|   3	| -	| 3-connected Gouraud Line
 *      LINE_F4  | 8	|Flat	|   4	| -	| 4-connected Flat Line
 *      LINE_G4  |11	|Gouraud|   4	| -	| 4-connected Gouraud Line
 *      ---------+------+-------+-------+-------+------------------------
 *	BLK_FILL | 5	|Flat	|   1   |OFF	| Block Write
 *      ---------+------+-------+-------+-------+------------------------
 *      SPRT	 | 6	|Flat	|   1   |ON     | free size Sprite
 *      SPRT_16	 | 5	|Flat	|   1   |ON     | 16x16 Sprite
 *      SPRT_8	 | 5	|Flat	|   1   |ON     | 8x8 Sprite
 *      ---------+------+-------+-------+-------+------------------------
 *      TILE	 | 5	|Flat	|   1   |ON     | free size Sprite
 *      TILE_16	 | 4	|Flat	|   1   |ON     | 16x16 Sprite
 *      TILE_8	 | 4	|Flat	|   1   |ON     | 8x8 Sprite
 *      TILE_1	 | 4	|Flat	|   1   |ON     | 8x8 Sprite
 *      ---------+------+-------+-------+-------+------------------------
 *      DR_TWIN	 | 4	|   -	|   -   | -     | Texture Window
 *      DR_AREA	 | 4	|   -	|   -   | -     | Drawing Area
 *      DR_OFFSET| 4	|   -	|   -   | -     | Drawing Offset
 *      DR_PRIO  | 4	|   -	|   -   | -     | Drawing Priority
 *      DR_MODE  | 4	|   -	|   -   | -     | Drawing Mode
 *      DR_ENV   |17	|   -	|   -	| -	| Drawing Environment
 *      DR_MOVE  | 7	|   -	|   -	| -	| MoveImage
 *      DR_LOAD  |18	|   -	|   -	| -	| LoadImage
 *	DR_TPAGE | 3    |   -	|   -	| -	| Drawing TPage
 *
 *	*1: in long-word
 *
 * Texture Attributes:
 *	abr: ambient rate
 *		abr	0	1	2	3
 *		-------------------------------------
 *		Front	0.5	1.0	0.5	-1.0
 *		Back	0.5	1.0	1.0	 1.0
 *
 *	tp: texture mode
 *		 tp	0	1	2	
 *		 -----------------------------
 *		 depth	4bit	8bit	16bit
 *		 color	CLUT	CLUT	DIRECT
 */

/* This file must be included after windows.h */
#ifdef _INC_WINDOWS
#define RECT PS_RECT
#endif /* _INC_WINDOWS */

#ifdef _WINDOWS
#ifndef PASCAL
#  define PASCAL _pascal
#endif /* !PASCAL */
#ifndef FAR
#  define FAR _far
#endif /* !FAR */
#ifndef NEAR
#  define NEAR _near
#endif /* !NEAR */

#else

#ifndef PASCAL
#  define PASCAL
#endif /* !PASCAL */
#ifndef FAR
#  define FAR
#endif /* !FAR */
#ifndef NEAR
#  define NEAR
#endif /* !NEAR */
#endif	/* _WINDOWS */

/*
 *	Externals
 */
extern int (FAR _export *GPU_printf)();	/* printf() object */

#ifdef _INC_WINDOWS
#define GPU_printf FntPrint
#endif /* _INC_WINDOWS */

/*
 *	Time-out Cycle
 */
#define WAIT_TIME	0x800000

/*
 *	General Macros
 */
#define limitRange(x, l, h)	((x)=((x)<(l)?(l):(x)>(h)?(h):(x)))

/*
 *	Set/Add Vector/Rectangle Attributes
 */
#define setVector(v, _x, _y, _z) \
	(v)->vx = _x, (v)->vy = _y, (v)->vz = _z	

#define applyVector(v, _x, _y, _z, op) \
	(v)->vx op _x, (v)->vy op _y, (v)->vz op _z	

#define copyVector(v0, v1) \
	(v0)->vx = (v1)->vx, (v0)->vy = (v1)->vy, (v0)->vz = (v1)->vz 

#define addVector(v0, v1) \
	(v0)->vx += (v1)->vx,	\
	(v0)->vy += (v1)->vy,	\
	(v0)->vz += (v1)->vz	
	
#define dumpVector(str, v)	\
	GPU_printf("%s=(%d,%d,%d)\n", str, (v)->vx, (v)->vy, (v)->vz)

#define dumpMatrix(x)	\
	GPU_printf("\t%5d,%5d,%5d\n",(x)->m[0][0],(x)->m[0][1],(x)->m[0][2]),\
	GPU_printf("\t%5d,%5d,%5d\n",(x)->m[1][0],(x)->m[1][1],(x)->m[1][2]),\
	GPU_printf("\t%5d,%5d,%5d\n",(x)->m[2][0],(x)->m[2][1],(x)->m[2][2])

#define setRECT(r, _x, _y, _w, _h) \
	(r)->x = (_x),(r)->y = (_y),(r)->w = (_w),(r)->h = (_h)

/*
 *	Set Primitive Attributes
 */
#define setTPage(p,tp,abr,x,y) \
	((p)->tpage = GetTPage(tp,abr,x,y))

#define setClut(p,x,y) \
	((p)->clut = GetClut(x,y))
					   
/*
 * Set Primitive Colors
 */
#define setRGB0(p,_r0,_g0,_b0)						\
	(p)->r0 = _r0,(p)->g0 = _g0,(p)->b0 = _b0
	
#define setRGB1(p,_r1,_g1,_b1)						\
	(p)->r1 = _r1,(p)->g1 = _g1,(p)->b1 = _b1

#define setRGB2(p,_r2,_g2,_b2)						\
	(p)->r2 = _r2,(p)->g2 = _g2,(p)->b2 = _b2
	
#define setRGB3(p,_r3,_g3,_b3)						\
	(p)->r3 = _r3,(p)->g3 = _g3,(p)->b3 = _b3
	
/*
 * Set Primitive Screen Points
 */
#define setXY0(p,_x0,_y0)						\
	(p)->x0 = (_x0), (p)->y0 = (_y0)				\

#define setXY2(p,_x0,_y0,_x1,_y1)					\
	(p)->x0 = (_x0), (p)->y0 = (_y0),				\
	(p)->x1 = (_x1), (p)->y1 = (_y1)

#define setXY3(p,_x0,_y0,_x1,_y1,_x2,_y2)				\
	(p)->x0 = (_x0), (p)->y0 = (_y0),				\
	(p)->x1 = (_x1), (p)->y1 = (_y1),				\
	(p)->x2 = (_x2), (p)->y2 = (_y2)

#define setXY4(p,_x0,_y0,_x1,_y1,_x2,_y2,_x3,_y3) 			\
	(p)->x0 = (_x0), (p)->y0 = (_y0),				\
	(p)->x1 = (_x1), (p)->y1 = (_y1),				\
	(p)->x2 = (_x2), (p)->y2 = (_y2),				\
	(p)->x3 = (_x3), (p)->y3 = (_y3)

#define setXYWH(p,_x0,_y0,_w,_h)					\
	(p)->x0 = (_x0),      (p)->y0 = (_y0),				\
	(p)->x1 = (_x0)+(_w), (p)->y1 = (_y0),				\
	(p)->x2 = (_x0),      (p)->y2 = (_y0)+(_h),			\
	(p)->x3 = (_x0)+(_w), (p)->y3 = (_y0)+(_h)

/*
 * Set Primitive Width/Height
 */
#define setWH(p,_w,_h)	(p)->w = _w, (p)->h = _h

/*
 * Set Primitive Texture Points
 */
#define setUV0(p,_u0,_v0)						\
	(p)->u0 = (_u0), (p)->v0 = (_v0)				\
	
#define setUV3(p,_u0,_v0,_u1,_v1,_u2,_v2)				\
	(p)->u0 = (_u0), (p)->v0 = (_v0),				\
	(p)->u1 = (_u1), (p)->v1 = (_v1),				\
	(p)->u2 = (_u2), (p)->v2 = (_v2)
	
#define setUV4(p,_u0,_v0,_u1,_v1,_u2,_v2,_u3,_v3) 			\
	(p)->u0 = (_u0), (p)->v0 = (_v0),				\
	(p)->u1 = (_u1), (p)->v1 = (_v1),				\
	(p)->u2 = (_u2), (p)->v2 = (_v2),				\
	(p)->u3 = (_u3), (p)->v3 = (_v3)

#define setUVWH(p,_u0,_v0,_w,_h)					\
	(p)->u0 = (_u0),      (p)->v0 = (_v0),				\
	(p)->u1 = (_u0)+(_w), (p)->v1 = (_v0),				\
	(p)->u2 = (_u0),      (p)->v2 = (_v0)+(_h),			\
	(p)->u3 = (_u0)+(_w), (p)->v3 = (_v0)+(_h)

	
/*
 * Dump Primivie Parameters
 */
#define dumpRECT(r)	\
	GPU_printf("(%d,%d)-(%d,%d)\n", (r)->x,(r)->y,(r)->w,(r)->h)

#define dumpWH(p)	GPU_printf("(%d,%d)\n", (p)->w,  (p)->h ) 
#define dumpXY0(p)	GPU_printf("(%d,%d)\n", (p)->x0, (p)->y0) 
#define dumpUV0(p)	GPU_printf("(%d,%d)\n", (p)->u0, (p)->v0) 

#define dumpXY2(p)							\
	GPU_printf("(%d,%d)-(%d,%d)\n",					\
	(p)->x0, (p)->y0, (p)->x1, (p)->y1)

#define dumpXY3(p)							\
	GPU_printf("(%d,%d)-(%d,%d)-(%d,%d)\n",				\
	(p)->x0, (p)->y0, (p)->x1, (p)->y1,				\
	(p)->x2, (p)->y2)

#define dumpUV3(p)							\
	GPU_printf("(%d,%d)-(%d,%d)-(%d,%d)\n",				\
	(p)->u0, (p)->v0, (p)->u1, (p)->v1,				\
	(p)->u2, (p)->v2)

#define dumpXY4(p)							\
	GPU_printf("(%d,%d)-(%d,%d)-(%d,%d)-(%d,%d)\n",			\
	(p)->x0, (p)->y0, (p)->x1, (p)->y1,				\
	(p)->x2, (p)->y2, (p)->x3, (p)->y3)

#define dumpUV4(p)							\
	GPU_printf("(%d,%d)-(%d,%d)-(%d,%d)-(%d,%d)\n",			\
	(p)->u0, (p)->v0, (p)->u1, (p)->v1,				\
	(p)->u2, (p)->v2, (p)->u3, (p)->v3)			

#define dumpRGB0(p)							\
	GPU_printf("(%3d,%3d,%3d)\n", (p)->r0, (p)->g0, (p)->b0) 	
		   
#define dumpRGB1(p)							\
	GPU_printf("(%3d,%3d,%3d)\n", (p)->r1, (p)->g1, (p)->b1)	
		   
#define dumpRGB2(p)							\
	GPU_printf("(%3d,%3d,%3d)\n", (p)->r2, (p)->g2, (p)->b2) 
		   
#define dumpRGB3(p)							\
	GPU_printf("(%3d,%3d,%3d)\n", (p)->r3, (p)->g3, (p)->b3) 	

/*
 * Primitive Handling Macros
 */
#define setlen( p, _len) 	(((P_TAG FAR*)(p))->len  = (u_char)(_len))
#define setaddr(p, _addr)	(((P_TAG FAR*)(p))->addr = (u_long)(_addr))
#define setcode(p, _code)	(((P_TAG FAR*)(p))->code = (u_char)(_code))

#define getlen(p)    		(u_char)(((P_TAG FAR*)(p))->len)
#define getcode(p)   		(u_char)(((P_TAG FAR*)(p))->code)
#define getaddr(p)   		(u_long)(((P_TAG FAR*)(p))->addr)

#ifndef _WINDOWS
#define nextPrim(p)  		(void *)((((P_TAG *)(p))->addr)|0x80000000)
#define isendprim(p) 		((((P_TAG *)(p))->addr)==0xffffff)
#else
#define nextPrim(p)  		(void FAR*)((((P_TAG FAR*)(p))->addr))
#define isendprim(p) 		((((P_TAG FAR*)(p))->addr)==0xffffffff)
#endif /* !_WINDOWS */

#define addPrim(ot, p)		setaddr(p, getaddr(ot)), setaddr(ot, p)
#define addPrims(ot, p0, p1)	setaddr(p1, getaddr(ot)),setaddr(ot, p0)

#define catPrim(p0, p1)		setaddr(p0, p1)
#define termPrim(p)		setaddr(p, 0xffffffff)

#define setSemiTrans(p, abe) \
	((abe)?setcode(p, getcode(p)|0x02):setcode(p, getcode(p)&~0x02))

#define setShadeTex(p, tge) \
	((tge)?setcode(p, getcode(p)|0x01):setcode(p, getcode(p)&~0x01))

#define getTPage(tp, abr, x, y) 					\
	((GetGraphType()==1||GetGraphType()==2)?			\
	 ((((tp)&0x3)<<9)|(((abr)&0x3)<<7)|(((y)&0x300)>>3)|(((x)&0x3ff)>>6)):\
	 ((((tp)&0x3)<<7)|(((abr)&0x3)<<5)|(((y)&0x100)>>4)|(((x)&0x3ff)>>6)| \
	 (((y)&0x200)<<2)))

#define getClut(x, y) \
	((y<<6)|((x>>4)&0x3f))

#define dumpTPage(tpage)						\
	((GetGraphType()==1||GetGraphType()==2)?			\
	GPU_printf("tpage: (%d,%d,%d,%d)\n",				\
			   ((tpage)>>9)&0x003,((tpage)>>7)&0x003,	\
			   ((tpage)<<6)&0x7c0,((tpage)<<3)&0x300):	\
	GPU_printf("tpage: (%d,%d,%d,%d)\n",				\
			   ((tpage)>>7)&0x003,((tpage)>>5)&0x003,	\
			   ((tpage)<<6)&0x7c0,				\
			   (((tpage)<<4)&0x100)+(((tpage)>>2)&0x200)))


#define dumpClut(clut) \
	GPU_printf("clut: (%d,%d)\n", (clut&0x3f)<<4, (clut>>6))

#define _get_mode(dfe, dtd, tpage)	\
	((GetGraphType()==1||GetGraphType()==2)?	\
		((0xe1000000)|((dtd)?0x0800:0)| \
		((dfe)?0x1000:0)|((tpage)&0x27ff)):	\
		((0xe1000000)|((dtd)?0x0200:0)| \
		((dfe)?0x0400:0)|((tpage)&0x9ff)))

#define setDrawTPage(p, dfe, dtd, tpage)	\
	setlen(p, 1),	\
	((u_long *)(p))[1] = _get_mode(dfe, dtd, tpage)

/*	Primitive 	Lentgh		Code				*/
/*--------------------------------------------------------------------	*/
/*									*/
#define setPolyF3(p)	setlen(p, 4),  setcode(p, 0x20)
#define setPolyFT3(p)	setlen(p, 7),  setcode(p, 0x24)
#define setPolyG3(p)	setlen(p, 6),  setcode(p, 0x30)
#define setPolyGT3(p)	setlen(p, 9),  setcode(p, 0x34)
#define setPolyF4(p)	setlen(p, 5),  setcode(p, 0x28)
#define setPolyFT4(p)	setlen(p, 9),  setcode(p, 0x2c)
#define setPolyG4(p)	setlen(p, 8),  setcode(p, 0x38)
#define setPolyGT4(p)	setlen(p, 12), setcode(p, 0x3c)

#define setSprt8(p)	setlen(p, 3),  setcode(p, 0x74)
#define setSprt16(p)	setlen(p, 3),  setcode(p, 0x7c)
#define setSprt(p)	setlen(p, 4),  setcode(p, 0x64)

#define setTile1(p)	setlen(p, 2),  setcode(p, 0x68)
#define setTile8(p)	setlen(p, 2),  setcode(p, 0x70)
#define setTile16(p)	setlen(p, 2),  setcode(p, 0x78)
#define setTile(p)	setlen(p, 3),  setcode(p, 0x60)
#define setBlockFill(p)	setlen(p, 3),  setcode(p, 0x02)
#define setLineF2(p)	setlen(p, 3),  setcode(p, 0x40)
#define setLineG2(p)	setlen(p, 4),  setcode(p, 0x50)
#define setLineF3(p)	setlen(p, 5),  setcode(p, 0x48),(p)->pad   = 0x55555555
#define setLineG3(p)	setlen(p, 7),  setcode(p, 0x58),(p)->pad   = 0x55555555
#define setLineF4(p)	setlen(p, 6),  setcode(p, 0x4c),(p)->pad   = 0x55555555
#define setLineG4(p)	setlen(p, 9),  setcode(p, 0x5c),(p)->pad   = 0x55555555
#define setDrawMove(p)	setlen(p, 5),  setcode(p, 0x01),(p)->code2 = 0x80000000
	
/*
 * Rectangle:
 */
typedef struct _RECT {
	short x, y;		/* offset point on VRAM */
	short w, h;		/* width and height */
} RECT;

#ifndef M_I86
typedef struct {
	int x, y;		/* offset point on VRAM */
	int w, h;		/* width and height */
} RECT32;
#else
typedef struct {
	long x, y;		/* offset point on VRAM */
	long w, h;		/* width and height */
} RECT32;
#endif

/*
 * Environment 
 */
typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_long	code[15];
} DR_ENV;				/* Packed Drawing Environment */
	       
typedef struct {
	RECT	clip;		/* clip area */
	short	ofs[2];		/* drawing offset */
	RECT	tw;		/* texture window */
	u_short tpage;		/* texture page */	
	u_char	dtd;		/* dither flag (0:off, 1:on) */
	u_char	dfe;		/* flag to draw on display area (0:off 1:on) */
	u_char	isbg;		/* enable to auto-clear */
	u_char	r0, g0, b0;	/* initital background color */
	DR_ENV	dr_env;		/* reserved */
} DRAWENV;
	       
typedef struct {
	RECT	disp;		/* display area */
	RECT	screen;		/* display start point */
	u_char	isinter;	/* interlace 0: off 1: on */
	u_char	isrgb24;	/* RGB24 bit mode */
	u_char	pad0, pad1;	/* reserved */
} DISPENV;

/*
 * Polygon Primitive Definitions
 */
typedef struct {
#ifndef _WINDOWS
	unsigned	addr: 24;
	unsigned 	len:   8;
#else
	u_long		addr;
	u_long		len;
#endif /* _WINDOWS */
	u_char		r0, g0, b0, code;
} P_TAG;

#ifdef _WINDOWS
typedef struct {
	u_long	addr;
	u_long	len;
} OT;
#else
#define OT u_long
#endif /* _WINDOWS */

typedef struct {
	u_char	r0, g0, b0, code;
} P_CODE;
	
typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	short	x1,	y1;
	short	x2,	y2;
} POLY_F3;				/* Flat Triangle */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	short	x1,	y1;
	short	x2,	y2;
	short	x3,	y3;
} POLY_F4;				/* Flat Quadrangle */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	u_char	u0, v0;	u_short	clut;
	short	x1,	y1;
	u_char	u1, v1;	u_short	tpage;
	short	x2,	y2;
	u_char	u2, v2;	u_short	pad1;
} POLY_FT3;				/* Flat Textured Triangle */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	u_char	u0, v0;	u_short	clut;
	short	x1,	y1;
	u_char	u1, v1;	u_short	tpage;
	short	x2,	y2;
	u_char	u2, v2;	u_short	pad1;
	short	x3,	y3;
	u_char	u3, v3;	u_short	pad2;
} POLY_FT4;				/* Flat Textured Quadrangle */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	u_char	r1, g1, b1, pad1;
	short	x1,	y1;
	u_char	r2, g2, b2, pad2;
	short	x2,	y2;
} POLY_G3;				/* Gouraud Triangle */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	u_char	r1, g1, b1, pad1;
	short	x1,	y1;
	u_char	r2, g2, b2, pad2;
	short	x2,	y2;
	u_char	r3, g3, b3, pad3;
	short	x3,	y3;
} POLY_G4;				/* Gouraud Quadrangle */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	u_char	u0, v0;	u_short	clut;
	u_char	r1, g1, b1, p1;
	short	x1,	y1;
	u_char	u1, v1;	u_short	tpage;
	u_char	r2, g2, b2, p2;
	short	x2,	y2;
	u_char	u2, v2;	u_short	pad2;
} POLY_GT3;				/* Gouraud Textured Triangle */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	u_char	u0, v0;	u_short	clut;
	u_char	r1, g1, b1, p1;
	short	x1,	y1;
	u_char	u1, v1;	u_short	tpage;
	u_char	r2, g2, b2, p2;
	short	x2,	y2;
	u_char	u2, v2;	u_short	pad2;
	u_char	r3, g3, b3, p3;
	short	x3,	y3;
	u_char	u3, v3;	u_short	pad3;
} POLY_GT4;				/* Gouraud Textured Quadrangle */

/*
 * Line Primitive Definitions
 */
typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	short	x1,	y1;
} LINE_F2;				/* Unconnected Flat Line */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	u_char	r1, g1, b1, p1;
	short	x1,	y1;
} LINE_G2;				/* Unconnected Gouraud Line */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	short	x1,	y1;
	short	x2,	y2;
	u_long	pad;
} LINE_F3;				/* 2 connected Flat Line */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	u_char	r1, g1, b1, p1;
	short	x1,	y1;
	u_char	r2, g2, b2, p2;
	short	x2,	y2;
	u_long	pad;
} LINE_G3;				/* 2 connected Gouraud Line */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	short	x1,	y1;
	short	x2,	y2;
	short	x3,	y3;
	u_long	pad;
} LINE_F4;				/* 3 connected Flat Line Quadrangle */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	u_char	r1, g1, b1, p1;
	short	x1,	y1;
	u_char	r2, g2, b2, p2;
	short	x2,	y2;
	u_char	r3, g3, b3, p3;
	short	x3,	y3;
	u_long	pad;
} LINE_G4;				/* 3 connected Gouraud Line */

/*
 * Block Fill Primitive Definitions
 */
typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	u_short	x0, 	y0;
	u_short	w, 	h;	
} BLK_FILL;				/* Clear Frame Buffer */
	       
/*
 * Sprite Primitive Definitions
 */
typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	u_char	u0, v0;	u_short	clut;
	short	w,	h;
} SPRT;					/* free size Sprite */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	u_char	u0, v0;	u_short	clut;
} SPRT_16;				/* 16x16 Sprite */
	       
typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	u_char	u0, v0;	u_short	clut;
} SPRT_8;				/* 8x8 Sprite */
	       
/*
 * Tile Primitive Definitions
 */
typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
	short	w,	h;
} TILE;					/* free size Tile */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
} TILE_16;				/* 16x16 Tile */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
} TILE_8;				/* 8x8 Tile */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_char	r0, g0, b0, code;
	short	x0, 	y0;
} TILE_1;				/* 1x1 Tile */

/*
 *  Special Primitive Definitions
 */
typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_long	code[2];
} DR_MODE;				/* Drawing Mode */

typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_long	code[2];
} DR_PRIO;				/* Priority */
	       
typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_long	code[2];
} DR_TWIN;				/* Texture Window */
	       
typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_long	code[2];
} DR_AREA;				/* Drawing Area */
	       
typedef struct {
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_long	code[2];
} DR_OFFSET;				/* Drawing Offset */
	       
typedef struct {			/* MoveImage */
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_long	code[5];
} DR_MOVE;

typedef struct {			/* LoadImage */
	u_long	tag;
#ifdef M_I86
	u_long	len;
#endif /* M_I86 */
	u_long	code[3];
	u_long	p[13];
} DR_LOAD;

typedef	struct {
	u_long	tag;
	u_long	code[1];
} DR_TPAGE;				/* Drawing TPage */

/*
 *	Font Stream Parameters
 */
#define FNT_MAX_ID	8	/* max number of stream ID */
#define FNT_MAX_SPRT	1024	/* max number of sprites in all streams */

/*
 *	Multi-purpose Sony-TMD primitive
 */
typedef struct {
	u_long	id;	
	u_char	r0, g0, b0, p0;		/* Color of vertex 0 */
	u_char	r1, g1, b1, p1;		/* Color of vertex 1 */
	u_char	r2, g2, b2, p2;		/* Color of vertex 2 */
	u_char	r3, g3, b3, p3;		/* Color of vertex 3 */
	u_short	tpage, clut;		/* texture page ID, clut ID */
	u_char	u0, v0, u1, v1;		/* texture corner point */
	u_char	u2, v2, u3, v3;
	
	/* independent vertex model */
	SVECTOR	x0, x1, x2, x3;		/* 3D corner point */
	SVECTOR	n0, n1, n2, n3;		/* 3D corner normal vector */
	
	/* Common vertex model */
	SVECTOR	FAR *v_ofs;			/* offset to vertex database */
	SVECTOR	FAR *n_ofs;			/* offset to normal database */
	
	u_short	vert0, vert1; 		/* index of vertex */
	u_short	vert2, vert3;		
	u_short	norm0, norm1; 		/* index of normal */
	u_short	norm2, norm3;	
} TMD_PRIM;

/*
 *	Multi-purpose TIM image
 */
typedef struct {
	u_long  mode;		/* pixel mode */
	RECT	*crect;		/* CLUT rectangle on frame buffer */
	u_long	*caddr;		/* CLUT address on main memory */
	RECT	*prect;		/* texture image rectangle on frame buffer */
	u_long	*paddr;		/* texture image address on main memory */
} TIM_IMAGE;
	       
/*
 * Prototypes
 */
#if defined(_LANGUAGE_C_PLUS_PLUS)||defined(__cplusplus)||defined(c_plusplus)
extern "C" {
#ifndef _FNTPRINT_
#define _FNTPRINT_
int FAR _export FntPrint(void*,...);
#endif /* _FNTPRINT_ */
#ifndef _KANJIFNTPRINT_
#define _KANJIFNTPRINT_
int FAR _export KanjiFntPrint(void*,...);
#endif /* _KANJIFNTPRINT_ */
#else
#ifndef _FNTPRINT_
#define _FNTPRINT_
int FAR FntPrint(void*, ...);
#endif /* _FNTPRINT_ */
#ifndef _KANJIFNTPRINT_
#define _KANJIFNTPRINT_
int FAR KanjiFntPrint(void*, ...);
#endif /* _KANJIFNTPRINT_ */
#endif
	
DISPENV FAR* PASCAL _export GetDispEnv(DISPENV FAR *env);
DISPENV FAR* PASCAL _export PutDispEnv(DISPENV FAR *env);
DISPENV FAR* PASCAL _export SetDefDispEnv(DISPENV FAR *env, int x, int y, int w, int h);
DRAWENV FAR* PASCAL _export GetDrawEnv(DRAWENV FAR *env);
DRAWENV FAR* PASCAL _export PutDrawEnv(DRAWENV FAR *env);
DRAWENV FAR* PASCAL _export SetDefDrawEnv(DRAWENV FAR *env, int x, int y, int w, int h);
TIM_IMAGE _huge* PASCAL _export ReadTIM(TIM_IMAGE FAR *timimg);
TMD_PRIM FAR* PASCAL _export ReadTMD(TMD_PRIM FAR *tmdprim);
int FAR PASCAL _export CheckPrim(char FAR *s, u_long FAR *p);
int FAR PASCAL _export ClearImage(RECT FAR *rect, u_char r, u_char g, u_char b);
int FAR PASCAL _export DrawSync(int mode);
int FAR PASCAL _export FntOpen(int x, int y, int w, int h, int isbg, int n);
int FAR PASCAL _export GetGraphDebug(void) ;
int FAR PASCAL _export GetTimSize(u_char FAR *sjis);
int FAR PASCAL _export IsEndPrim(void FAR *p) ;
int FAR PASCAL _export KanjiFntOpen(int x, int y, int w, int h, int dx, int dy, int cx, int cy, int isbg, int n);
int FAR PASCAL _export Krom2Tim(u_char FAR *sjis, u_long FAR *taddr, int dx, int dy, int cdx, int cdy, u_int fg, u_int bg);
int FAR PASCAL _export LoadImage(RECT FAR *rect, u_long FAR *p);
int FAR PASCAL _export MargePrim(void FAR *p0, void FAR *p1);
int FAR PASCAL _export MoveImage(RECT FAR *rect, int x, int y);
int FAR PASCAL _export OpenTIM(u_long _huge *addr);
int FAR PASCAL _export OpenTMD(u_long _huge *tmd, int obj_no);
int FAR PASCAL _export PutKanji(u_char FAR *sjis);
int FAR PASCAL _export ResetGraph(int mode);
int FAR PASCAL _export SetGraphDebug(int level);
int FAR PASCAL _export SetGraphQueue(int mode);
int FAR PASCAL _export SetGraphReverse(int mode);
int FAR PASCAL _export StoreImage(RECT FAR *rect, u_long FAR *p);
extern int hankanjitrans(u_char scode);
extern int kanjitrans(u_short scode);
OT FAR* PASCAL _export ClearOTag(OT FAR *ot, int n);
OT FAR* PASCAL _export ClearOTagR(OT FAR *ot, int n);
u_long FAR* PASCAL _export FntFlush(int id);
u_long FAR* PASCAL _export KanjiFntFlush(int id);
u_long FAR PASCAL _export DrawSyncCallback(void (*func)());
u_short FAR PASCAL _export GetClut(int x, int y) ;
u_short FAR PASCAL _export GetTPage(int tp, int abr, int x, int y) ;
u_short FAR PASCAL _export LoadClut(u_long FAR *clut, int x, int y);
u_short FAR PASCAL _export LoadTPage(u_long FAR *pix, int tp, int abr, int x, int y, int w, int h);
void FAR* PASCAL _export NextPrim(void FAR *p) ;
void FAR PASCAL _export AddPrim(void FAR *ot, void FAR *p) ;
void FAR PASCAL _export AddPrims(void FAR *ot, void FAR *p0, void FAR *p1) ;
void FAR PASCAL _export CatPrim(void FAR *p0, void FAR *p1) ;
void FAR PASCAL _export DrawOTag(OT FAR *p);
void FAR PASCAL _export DrawOTagIO(OT FAR *p);
void FAR PASCAL _export DrawOTagEnv(OT FAR *p, DRAWENV FAR *env);
void FAR PASCAL _export DrawPrim(void FAR *p);
void FAR PASCAL _export DumpClut(u_short clut) ;
void FAR PASCAL _export DumpDispEnv(DISPENV FAR *env);
void FAR PASCAL _export DumpDrawEnv(DRAWENV FAR *env);
void FAR PASCAL _export DumpOTag(OT FAR *p);
void FAR PASCAL _export DumpTPage(u_short tpage) ;
void FAR PASCAL _export FntLoad(int tx, int ty);
void FAR PASCAL _export KanjiFntLoad(int tx, int ty);
void FAR PASCAL _export SetDispMask(int mask);
void FAR PASCAL _export SetDrawArea(DR_AREA FAR *p, RECT FAR *r);
void FAR PASCAL _export SetDrawEnv(DR_ENV FAR *dr_env, DRAWENV FAR *env);
void FAR PASCAL _export SetDrawLoad(DR_LOAD FAR *p, RECT FAR *rect);
void FAR PASCAL _export SetDrawMode(DR_MODE FAR *p, int dfe, int dtd, int tpage, RECT FAR* tw);
void FAR PASCAL _export SetDrawTPage(DR_TPAGE FAR *p, int dfe, int dtd, int tpage);
void FAR PASCAL _export SetDrawMove(DR_MOVE FAR *p, RECT FAR *rect, int x, int y) ;
void FAR PASCAL _export SetDrawOffset(DR_OFFSET FAR *p, u_short FAR *ofs);
void FAR PASCAL _export SetDumpFnt(int id);
void FAR PASCAL _export SetLineF2(LINE_F2 FAR *p) ;
void FAR PASCAL _export SetLineF3(LINE_F3 FAR *p) ;
void FAR PASCAL _export SetLineF4(LINE_F4 FAR *p) ;
void FAR PASCAL _export SetLineG2(LINE_G2 FAR *p) ;
void FAR PASCAL _export SetLineG3(LINE_G3 FAR *p) ;
void FAR PASCAL _export SetLineG4(LINE_G4 FAR *p) ;
void FAR PASCAL _export SetPolyF3(POLY_F3 FAR *p) ;
void FAR PASCAL _export SetPolyF4(POLY_F4 FAR *p) ;
void FAR PASCAL _export SetPolyFT3(POLY_FT3 FAR *p) ;
void FAR PASCAL _export SetPolyFT4(POLY_FT4 FAR *p) ;
void FAR PASCAL _export SetPolyG3(POLY_G3 FAR *p) ;
void FAR PASCAL _export SetPolyG4(POLY_G4 FAR *p) ;
void FAR PASCAL _export SetPolyGT3(POLY_GT3 FAR *p) ;
void FAR PASCAL _export SetPolyGT4(POLY_GT4 FAR *p) ;
void FAR PASCAL _export SetPriority(DR_PRIO FAR *p, int pbc, int pbw);
void FAR PASCAL _export SetSemiTrans(void FAR *p, int abe) ;
void FAR PASCAL _export SetShadeTex(void FAR *p, int tge) ;
void FAR PASCAL _export SetSprt(SPRT FAR *p) ;
void FAR PASCAL _export SetSprt16(SPRT_16 FAR *p) ;
void FAR PASCAL _export SetSprt8(SPRT_8 FAR *p) ;
void FAR PASCAL _export SetTexWindow(DR_TWIN FAR *p, RECT FAR *tw);
void FAR PASCAL _export SetTile(TILE FAR *p) ;
void FAR PASCAL _export SetTile1(TILE_1 FAR *p) ;
void FAR PASCAL _export SetTile16(TILE_16 FAR *p) ;
void FAR PASCAL _export SetTile8(TILE_8 FAR *p) ;
void FAR PASCAL _export TermPrim(void FAR *p) ;
extern u_long FAR* PASCAL _export BreakDraw(void);
extern void FAR PASCAL _export ContinueDraw(u_long FAR *insaddr, u_long FAR *contaddr);

#ifdef _WINDOWS
/* VSync: only mode==0 and mode>1 are available. */
long FAR PASCAL _export VSync(int mode);

/* SetPortId: Sets the artist board(DH-200) port address. */
void FAR PASCAL	_export SetPortId(int port);
/* AssertAboard: Detects the artist board at the address given by SetPortId(); returns 1 when it exists. */
int FAR PASCAL	_export AssertAboard();
/* OpenTMD2:  Give the arrays of vertices and normals before use ReadTMD2. */
int FAR PASCAL _export OpenTMD2(u_long FAR *tmd_head, int obj_no, SVECTOR FAR *vertices, SVECTOR FAR *normals);
/* ReadTMD2 allows to give each packet one by one while ReadTMD requires the whole TMD on memory. */
TMD_PRIM FAR* PASCAL _export ReadTMD2(u_long FAR *packet, TMD_PRIM FAR *tmdprim);
#endif /* _WINDOWS */
#if defined(_LANGUAGE_C_PLUS_PLUS)||defined(__cplusplus)||defined(c_plusplus)
}
#endif
#endif /* _LIBGPU_H_ */
