/* $PSLibId$ */
#ifndef _LIBETC_H_
#define _LIBETC_H_

/*
 *  (C) Copyright 1994-1996 Sony Computer Entertainment Inc. All Rights Reserved.
 *
 * 			libetc.h: Pad emulation functions for MS VC++.
 *				      Callbacks are not available.
 *
 */

#ifdef _WINDOWS
#include <types.h>
#endif /* _WINDOWS */
extern int PadIdentifier;
/*
 * PAD I/O (SIO Pad)
 */
#define PADLup     (1<<12)
#define PADLdown   (1<<14)
#define PADLleft   (1<<15)
#define PADLright  (1<<13)
#define PADRup     (1<< 4)
#define PADRdown   (1<< 6)
#define PADRleft   (1<< 7)
#define PADRright  (1<< 5)
#define PADi       (1<< 9)
#define PADj       (1<<10)
#define PADk       (1<< 8)
#define PADl       (1<< 3)
#define PADm       (1<< 1)
#define PADn       (1<< 2)
#define PADo       (1<< 0)
#define PADh       (1<<11)
#define PADL1      PADn
#define PADL2      PADo
#define PADR1      PADl
#define PADR2      PADm
#define PADstart   PADh
#define PADselect  PADk
		    
#define MOUSEleft  (1<<3)
#define MOUSEright (1<<2)

/*
 * PAD utility macro: _PAD(x,y)
 *   x: controller ID (0 or 1)
 *   y: PAD assign macro
 *
 * Example: _PAD(0,PADstart) ... PADstart of controller 1
 *          _PAD(1,PADLup)   ... PADLup of controller 2
 */
#define _PAD(x,y) ((y)<<((x)<<4))

#ifndef _WINDOWS /* N/A for Windows */
/* scratch pad address 0x1f800000 - 0x1f800400 */
#define getScratchAddr(offset)  ((u_long *)(0x1f800000+offset*4))
#endif
/*
 * Video Mode:	NTSC/PAL
 */
#define MODE_NTSC 0
#define MODE_PAL 1 

/*
 * Prototypes
 */

#if defined(_LANGUAGE_C_PLUS_PLUS)||defined(__cplusplus)||defined(c_plusplus)
extern "C" {
#endif
int CheckCallback(void);
void PadInit(int mode);
int ResetCallback(void);
int RestartCallback(void);
int StopCallback(void);
long VSync(int mode);
int VSyncCallback(void (FAR*f)());
int VSyncCallbacks(int ch, void (FAR*f)());
int GetVideoMode (void);
long SetVideoMode (long mode);
unsigned long PadRead(int id);
void PadStop(void);

#ifdef _WINDOWS
#define VK_MAX 256
extern int PadKeyMap[VK_MAX];
void PadSet(unsigned long status);
unsigned long KeyDown(unsigned int key);
unsigned long KeyUp(unsigned int key);
int GetPortId();

#ifdef _INC_WINDOWS
long PadReadProc (HWND hwnd, UINT message, UINT wParam, LONG lParam);
#endif /* _INC_WINDOWS */

#endif /* _WINDOWS */

#if defined(_LANGUAGE_C_PLUS_PLUS)||defined(__cplusplus)||defined(c_plusplus)
}
#endif

/* *** default map initialized by PadInit(1) ***
 * PadKeyMap[VK_MAX] = {
 *	0,				/*                     0x00 *
 *	0,				/* VK_LBUTTON          0x01 *
 *	0,		 		/* VK_RBUTTON          0x02 *
 *	0,				/* VK_CANCEL           0x03 *
 *	0, 				/* VK_MBUTTON          0x04 *
 *	0, 				/*                     0x05 *
 *	0, 				/*                     0x06 *
 *	0, 				/*                     0x07 *
 *	0, 				/* VK_BACK             0x08 *
 *	0, 				/* VK_TAB              0x09 *
 *	0, 				/*                     0x0A *
 *	0, 				/*                     0x0B *
 *	0,				/* VK_CLEAR            0x0C *
 *	PADstart,			/* VK_RETURN           0x0D *
 *	0, 				/*                     0x0E *
 *	0, 				/*                     0x0F *
 *	0,				/* VK_SHIFT            0x10 *
 *	0,				/* VK_CONTROL          0x11 *@
 *	0,				/* VK_MENU             0x12 *
 *	0,				/* VK_PAUSE            0x13 *
 *	0,				/* VK_CAPITAL          0x14 *
 *	0,				/* VK_KANA             0x15 *
 *	0, 				/*                     0x16 *
 *	0, 				/*                     0x17 *
 *	0, 				/*                     0x18 *
 *	0,				/* VK_KANJI            0x19 *
 *	0, 				/*                     0x1A *
 *	PADselect,			/* VK_ESCAPE           0x1B *
 *	0,				/* VK_CONVERT          0x1C *
 *	0,				/* VK_NONCONVERT       0x1D *
 *	0, 				/*                     0x1E *
 *	0, 				/*                     0x1F *
 *	0,				/* VK_SPACE            0x20 *
 *	0,				/* VK_PRIOR            0x21 *
 *	0,				/* VK_NEXT             0x22 *
 *	0,				/* VK_END              0x23 *
 *	0,				/* VK_HOME             0x24 *
 *	PADLleft,			/* VK_LEFT             0x25 *
 *	PADLup,				/* VK_UP               0x26 *
 *	PADLright,			/* VK_RIGHT            0x27 *
 *	PADLdown,			/* VK_DOWN             0x28 *
 *	0,				/* VK_SELECT           0x29 *
 *	0,				/* VK_PRINT            0x2A *
 *	0,				/* VK_EXECUTE          0x2B *
 *	0,				/* VK_SNAPSHOT         0x2C *
 *	0,				/* VK_INSERT           0x2D *
 *	0,				/* VK_DELETE           0x2E *
 *	0,				/* VK_HELP             0x2F *
 *	0,				/* VK_0                0x30 *
 *	0,				/* VK_1                0x31 *
 *	0,				/* VK_2                0x32 *
 *	0,				/* VK_3                0x33 *
 *	0,				/* VK_4                0x34 *
 *	0,				/* VK_5                0x35 *
 *	0,				/* VK_6                0x36 *
 *	0,				/* VK_7                0x37 *
 *	0,				/* VK_8                0x38 *
 *	0,				/* VK_9                0x39 *
 *	0,				/*                     0x3A *
 *	0,				/*                     0x3B *
 *	0,				/*                     0x3C *
 *	0,				/*                     0x3D *
 *	0,				/*                     0x3E *
 *	0,				/*                     0x3F *
 *	0,				/*                     0x40 *		
 *	0,				/* 'A'                 0x41 *
 *	0,				/* 'B'	               0x42 *
 *	PADLdown,			/* 'C'	               0x43 *
 *	0,				/* 'D'	               0x44 *
 *	PADLup,				/* 'E'	               0x45 *
 *	PADLright,			/* 'F'	               0x46 *
 *	PADselect,			/* 'G'	               0x47 *
 *	PADstart,			/* 'H'	               0x48 *
 *	PADRup,				/* 'I'	               0x49 *
 *	PADRleft,			/* 'J'	               0x4A *
 *	0,				/* 'K'	               0x4B *
 *	PADRright,			/* 'L'	               0x4C *
 *	PADRdown,			/* 'M'	               0x4D *
 *	0,				/* 'N'	               0x4E *
 *	0,				/* 'O'	               0x4F *
 *	0,				/* 'P'	               0x50 *
 *	0,				/* 'Q'	               0x51 *
 *	PADL1,				/* 'R'	               0x52 *
 *	PADLleft,			/* 'S'	               0x53 *
 *	PADL2,				/* 'T'	               0x54 *
 *	PADR1,				/* 'U'	               0x55 *
 *	0,				/* 'V'	               0x56 *
 *	0,				/* 'W'                 0x57 *
 *	0,				/* 'X	               0x58 *
 *	PADR2,				/* 'Y'	               0x59 *
 *	0,				/* 'Z'	               0x5A *
 *	0,				/*                     0x5B *
 *	0,				/*                     0x5C *
 *	0,				/*                     0x5D *
 *	0,				/*                     0x5E *
 *	0,				/*                     0x5F *
 *	0,				/* VK_NUMPAD0          0x60 *
 *	0,				/* VK_NUMPAD1          0x61 *
 *	0,				/* VK_NUMPAD2          0x62 *
 *	0,				/* VK_NUMPAD3          0x63 *
 *	0,				/* VK_NUMPAD4          0x64 *
 *	0,				/* VK_NUMPAD5          0x65 *
 *	0,				/* VK_NUMPAD6          0x66 *
 *	0,				/* VK_NUMPAD7          0x67 *
 *	0,				/* VK_NUMPAD8          0x68 *
 *	0,				/* VK_NUMPAD9          0x69 *
 *	0,				/* VK_MULTIPLY         0x6A *
 *	0,				/* VK_ADD              0x6B *
 *	0,				/* VK_SEPARATOR        0x6C *
 *	0,				/* VK_SUBTRACT         0x6D *
 *	0,				/* VK_DECIMAL          0x6E *
 *	0,				/* VK_DIVIDE           0x6F *
 *	0,				/* VK_F1               0x70 *
 *	0,				/* VK_F2               0x71 *
 *	0,				/* VK_F3               0x72 *
 *	0,				/* VK_F4               0x73 *
 *	0,				/* VK_F5               0x74 *
 *	0,				/* VK_F6               0x75 *
 *	0,				/* VK_F7               0x76 *
 *	0,				/* VK_F8               0x77 *
 *	0,				/* VK_F9               0x78 *
 *	0,				/* VK_F10              0x79 *
 *	0,				/* VK_F11              0x7A *
 *	0,				/* VK_F12              0x7B *
 *	0,				/* VK_F13              0x7C *
 *	0,				/* VK_F14              0x7D *
 *	0,				/* VK_F15              0x7E *
 *	0,				/* VK_F16              0x7F *
 *	0,				/* VK_F17              0x80 *
 *	0,				/* VK_F18              0x81 *
 *	0,				/* VK_F19              0x82 *
 *	0,				/* VK_F20              0x83 *
 *	0,				/* VK_F21              0x84 *
 *	0,				/* VK_F22              0x85 *
 *	0,				/* VK_F23              0x86 *
 *	0,				/* VK_F24              0x87 *
 *	0,				/*                     0x88 *
 *	0,				/*                     0x89 *
 *	0,				/*                     0x8A *
 *	0,				/*                     0x8B *
 *	0,				/*                     0x8C *
 *	0,				/*                     0x8D *
 *	0,				/*                     0x8E *
 *	0,				/*                     0x8F *
 *	0,				/* VK_NUMLOCK          0x90 *
 *	0,				/* VK_SCROLL           0x91 *
 * };
 */

#endif /* _LIBETC_H_ */


