/*
 *	MIMeWave data viewer
 *
 *	"nmime.c"
 *
 *		Version 1.00	June 13, 1995
 *
 *		Copyright (C) 1995 by Sony Computer Entertainment Inc.
 *		All Rights Reserved.
 */

#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libgs.h>  
#define NMIMEMAIN
#include "nmime.h"  

/* object information of TMD file /* TMD file  IuWFNg  */
typedef struct {
	SVECTOR *vtxtop;	/* start address of vertex block */
	u_long vtxtotal;	/* total number of vertices */
	SVECTOR *nrmtop;	/* start address of normal block */
	u_long nrmtotal;	/* total number of normals */
	u_long *prmtop;		/* start address of primitive block (not used) */
	u_long prmtotal;	/* total number of primitives (not used) */
	u_long scale;		/* scale of object (not used) */
} TMDOBJECT;

/* VDF/NDF file (difference data file) */
typedef struct {
	SVECTOR		*top;	/* start address of different vectors */
	u_long		offset;	/* offset of different vectors */
	u_long		total;	/* total number of different vectors */
	u_long		object;	/* object number */
} MDFOBJECT;

/* structure of TMD information for MIMe operation */
typedef struct {
	u_long 		objtotal;	/* total number of objects in TMD */
	SVECTOR		*orgvtx;	/* address of original vertex buffer */
	SVECTOR		*orgnrm;	/* address of original normal buffer */
	TMDOBJECT 	*tmdobj;	/* address of TMD information array */
} TMDDATA;

/* structure of VDF/NDF information for MIMe operation */
typedef struct {
	u_long 		objtotal;	/* total number of objects in VDF/NDF */
	MDFOBJECT	*mdfvct;	/* address of object information array */	
	long		*mime;		/* address of MIMe Weight (Control) array */
} MDFDATA;

static void set_tmd_info(u_long *model, TMDDATA *tmddt);
static void original_vertex_save(TMDDATA *tmddt, SVECTOR *orgvtxbf);
static void original_normal_save(TMDDATA *tmddt, SVECTOR *orgnrmbf);
static void reset_mime_vertex(TMDDATA *tmddt);
static void reset_mime_norm(TMDDATA *tmddt);
static void set_mime_vertex(TMDDATA *tmddt, MDFDATA *vtxdt);
static void set_mime_norm(TMDDATA *tmddt, MDFDATA *nrmdt);
static void set_mdf_vertex(u_long *mdfdtvtx, MDFDATA *vtxdt);
static void set_mdf_normal(u_long *mdfdtnrm, MDFDATA *nrmdt);


TMDOBJECT tmdobj0[MIMEMODELMAX][OBJMAX]; /* TMD info array for MIMe
					 /* MIMevZpTMDIuWFNgz */
MDFOBJECT mdfvtx0[MIMEMODELMAX][MIMEMAX]; /* Vertex difference data
					  /* MIMevZp_f[^ */
MDFOBJECT mdfnrm0[MIMEMODELMAX][MIMEMAX]; /* Normal difference data
					  /* MIMevZp@f[^ */

/* Work buffers /* MIMevZpTMDf[^ */
TMDDATA tmddt0[MIMEMODELMAX];
MDFDATA vtxdt0[MIMEMODELMAX];
MDFDATA nrmdt0[MIMEMODELMAX];

/*==MIME Function==================*/

int init_mime_data(int id, u_long *modeladdr, u_long *mdfdatavtx, u_long *mdfdatanrm, u_long *orgvtxbuf, u_long *orgnrmbuf)
{
    int i;
/*    printf("in init_mime_data\n");*/

    tmddt0[id].objtotal=1;
    tmddt0[id].orgvtx=(SVECTOR *)orgvtxbuf;
    tmddt0[id].orgnrm=(SVECTOR *)orgnrmbuf;
    tmddt0[id].tmdobj=tmdobj0[id];
    vtxdt0[id].objtotal=1;
    vtxdt0[id].mdfvct=mdfvtx0[id];
    vtxdt0[id].mime= &mimepr[id][0];
    nrmdt0[id].objtotal=1;
    nrmdt0[id].mdfvct=mdfnrm0[id];
    nrmdt0[id].mime= &mimepr[id][0];

    set_tmd_info(modeladdr, &tmddt0[id]);	/* Set TMD data /* TMDf[^̃Zbg */

    if (mdfdatavtx!=NULL){
	/* Save the original vertex data /* IWi_̑ޔ */
	original_vertex_save(&tmddt0[id], (SVECTOR *)orgvtxbuf);
	/* Set the vertex difference data /* _f[^̃Zbg */
	set_mdf_vertex(mdfdatavtx,&vtxdt0[id]);
    }
    if (mdfdatanrm!=NULL){
        /* Save the original normal data /* IWi@̑ޔ */
	original_normal_save(&tmddt0[id], (SVECTOR *)orgnrmbuf); 
	/* Set the normal difference data /* @f[^̃Zbg */ 
	set_mdf_normal(mdfdatanrm,&nrmdt0[id]);
    }
    return vtxdt0[id].objtotal;
}

/* Reset and compute vertex MIMe /* _MIMevZ */
void vertex_mime(int id)
{
    reset_mime_vertex(&tmddt0[id]);		/* Revert to the original vertex data
						/* IWi_̕A@*/
    set_mime_vertex(&tmddt0[id],&vtxdt0[id]);	/* Compute vertex MIMe
						/* MIMevZi_j@*/
}

/* Reset and compute normal MIMe /* @MIMevZ */
void normal_mime(int id)
{
    reset_mime_norm(&tmddt0[id]);		/* Revert to the original normal data
						/* IWi@̕A@*/
    set_mime_norm(&tmddt0[id],&nrmdt0[id]);	/* Compute normal MIMe
						/* MIMevZi@j@*/
}


/* Set TMD data /* TMDf[^̃ZbeBO */
static void set_tmd_info(u_long *model, TMDDATA *tmddt)
{
    u_long size,*dop;
    int i,n;

    dop= model;
    dop++;   			/* Skip file header /* t@Cwb_̓ǂݔ΂ */
    dop++;   			/* Skip flag /* tO̓ǂݔ΂ */
    n = tmddt->objtotal = *dop;	/* Get the number of objects in TMD
				/* TMDf[^̃IuWFNg */
    dop++;
    for(i = 0; i < n; i++){
	tmddt->tmdobj[i].vtxtop = (SVECTOR *)*dop++ ;	/* pointer to the vertex data
							/* _̃|C^ */	
	tmddt->tmdobj[i].vtxtotal = *dop++ ;		/* total number of vertices
							/* _̑ */
	tmddt->tmdobj[i].nrmtop = (SVECTOR *)*dop++ ;	/* pointer to the normal data
							/* @̃|C^ */
	tmddt->tmdobj[i].nrmtotal = *dop++ ;		/* total number of normals
							/* @̑ */
	tmddt->tmdobj[i].prmtop = (u_long *)*dop++ ;	/* pointer to the polygon data
							/* |S|C^ */
	tmddt->tmdobj[i].prmtotal = *dop++ ;		/* total number of polygons
							/* |S̑ */
	tmddt->tmdobj[i].scale  = *dop++ ;		/* scale of object
							/* IuWFNg̃XP[ */
    } 

#ifdef VERBOSE
    printf("objtotal = %d\n", tmddt->objtotal);
    for(i = 0; i < n; i++){
	printf("v_top%d   = %x\n", 	i, 	tmddt->tmdobj[i].vtxtop );
	printf("v_num%d   = %d\n",	i,  	tmddt->tmdobj[i].vtxtotal );
	printf("nrm_top%d = %x\n",	i,	tmddt->tmdobj[i].nrmtop );
	printf("nrm_num%d = %d\n",	i, 	tmddt->tmdobj[i].nrmtotal );
	printf("prm_top%d = %x\n",	i, 	tmddt->tmdobj[i].prmtop );
	printf("prm_num%d = %d\n",	i, 	tmddt->tmdobj[i].prmtotal );
	printf("scale%d   = %d\n",	i, 	tmddt->tmdobj[i].scale );
    } 
#endif /* VERBOSE */
}

/* Set vertex difference data /* _f[^̃Zbg */
static void set_mdf_vertex(u_long *mdfdtvtx, MDFDATA *vtxdt)
{
    int i,n;
    u_long *dop2;

    dop2=mdfdtvtx;
    n = vtxdt->objtotal = *dop2++;

    dop2=mdfdtvtx;
    n = vtxdt->objtotal = *dop2++;

    for(i = 0; i < n; i++){
	vtxdt->mdfvct[i].object = *dop2++;	/* object number */ 		
	vtxdt->mdfvct[i].offset = *dop2++;	/* offset of differential vertex */ 
	vtxdt->mdfvct[i].total 	= *dop2++;	/* total number of differential vertex */
	vtxdt->mdfvct[i].top 	= (SVECTOR *)dop2; /* address of differential information */
	dop2 	+= vtxdt->mdfvct[i].total*2;
    }
}

/* Set normal differene data /* @f[^̃Zbg */
static void set_mdf_normal(u_long *mdfdtnrm, MDFDATA *nrmdt)
{
    int i,n;
    u_long  *dop2;


    dop2= mdfdtnrm;
    n = nrmdt->objtotal = *dop2++;
    for(i = 0; i < n; i++){
	nrmdt->mdfvct[i].object = *dop2++;	/* Object number
						/* IuWFNgԍ */ 	
	nrmdt->mdfvct[i].offset = *dop2++;	/* Offset of the different vertex */
						/* ̃ItZbg */ 	
	nrmdt->mdfvct[i].total 	= *dop2++;	/* number of different vertices
						/* ̑ */ 	 
	nrmdt->mdfvct[i].top 	= (SVECTOR *)dop2; /* Pointer to different vertices
						   /* ̃|C^ */
	dop2 	+= nrmdt->mdfvct[i].total*2;
    }
}


/* Save the original vertex data /* IWi_̑ޔ@*/
static void original_vertex_save(TMDDATA *tmddt, SVECTOR *orgvtxbf)
{
    SVECTOR *otp,*bsp,*dfp;
    int i,j,n,m;

    bsp= tmddt->orgvtx = orgvtxbf;
    m = tmddt->objtotal;
    n=0;
    for( j = 0; j < m; j++){
  	otp = tmddt->tmdobj[j].vtxtop;
	bsp += n;
	n = tmddt->tmdobj[j].vtxtotal;
        for( i = 0; i < n; i++) *(bsp+i) = *(otp+i);
    }
}

/* Save the original normal data /* IWi@̑ޔ@*/
static void original_normal_save(TMDDATA *tmddt, SVECTOR *orgnrmbf)
{
    SVECTOR *otp,*bsp,*bspb,*dfp;
    int i,j,n,m;

    bsp= tmddt->orgnrm = orgnrmbf;
    m = tmddt->objtotal;
    n=0;
    for( j = 0; j < m; j++){
  	otp = tmddt->tmdobj[j].nrmtop;
	bsp  += n;
	n = tmddt->tmdobj[j].nrmtotal;
        for( i = 0; i < n; i++) *(bsp+i) = *(otp+i);
    }
}

/* Revert to the original vertex data /* IWi_̕A@*/
static void reset_mime_vertex(TMDDATA *tmddt)
{
    SVECTOR *otp,*bsp;
    int i,j,n,m;

    bsp= tmddt->orgvtx;
    m = tmddt->objtotal;
    n=0;
    for( j = 0; j < m; j++){
  	otp = tmddt->tmdobj[j].vtxtop;
	bsp += n;
	n = tmddt->tmdobj[j].vtxtotal;
        for( i = 0; i < n; i++) *(otp+i) = *(bsp+i);
    }
}

/* Revert to the original normal data /* IWi@̕A@*/
static void reset_mime_norm(TMDDATA *tmddt)
{
    SVECTOR *otp,*bsp;
    int i,j,n,m;

    bsp= tmddt->orgnrm;
    m = tmddt->objtotal;
    n=0;
    for( j = 0; j < m; j++){
  	otp = tmddt->tmdobj[j].nrmtop;
	bsp += n;
	n = tmddt->tmdobj[j].nrmtotal;
        for( i = 0; i < n; i++) *(otp+i) = *(bsp+i);
    }
}

/* Compute vertex MIMe /*  MIMevZi_j@*/
static void set_mime_vertex(TMDDATA *tmddt, MDFDATA *vtxdt)
{
    SVECTOR *otp,*bsp,*dfp;
    int i,n;

    n = vtxdt->objtotal;

    for( i = 0; i < n; i++){
  	otp = tmddt->tmdobj[vtxdt->mdfvct[i].object].vtxtop+vtxdt->mdfvct[i].offset;
	dfp = vtxdt->mdfvct[i].top;
	if( vtxdt->mime[i] !=0 ) gteMIMefunc(otp,dfp, vtxdt->mdfvct[i].total,vtxdt->mime[i]);  
    }
}


/* Compute normal MIMe /* MIMevZi@j@*/
static void set_mime_norm(TMDDATA *tmddt, MDFDATA *nrmdt)
{
    SVECTOR *otp,*bsp,*dfp;
    VECTOR tmp;
    int i,n;

    n = nrmdt->objtotal;
    for( i = 0; i < n; i++){
  	otp = tmddt->tmdobj[nrmdt->mdfvct[i].object].nrmtop+nrmdt->mdfvct[i].offset;
	dfp = nrmdt->mdfvct[i].top;
	if( nrmdt->mime[i] !=0 ) 
	    {
	 	gteMIMefunc(otp,dfp, nrmdt->mdfvct[i].total,nrmdt->mime[i]);  
		tmp.vx = (otp+i)->vx;
 		tmp.vy = (otp+i)->vy;
		tmp.vz = (otp+i)->vz;
		VectorNormal(&tmp,&tmp); /* normalize /* K@*/
		(otp+i)->vx = tmp.vx;
 		(otp+i)->vy = tmp.vy;
		(otp+i)->vz = tmp.vz;
	    }
    }
}


/* Revert to the original vertex data */
void reset_mime_vdf(int id)
{
    reset_mime_vertex(&tmddt0[id]);
}

/* Revert to the original normal data */
void reset_mime_ndf(int id)
{
    reset_mime_norm(&tmddt0[id]);
}
