/*
 * 	Wave Linker: Converts WVF files into a binary file.
 *
 *		     Load the output file to the PlayStation board and 
 *	         run the sample program under the ..\preview directory.
 *
 *		Output file format: 
 *				0                          31
 *				____________________________
 *				|       no. of waves        |
 *				| no. of samples in wave[0] |
 *				|        sample[0]          |
 *				|        sample[1]          |
 *				|          ....	            |
 *				|        sample[n-1]        |
 *				| no. of samples in wave[1] |
 *				|        sample[0]          |
 *				|           ....            |
 *				|___________________________|
 *
 *		Copyright (C) 1995 Sony Computer Entertainment Inc.
 *				All Right Reserved.
 *
 */

#include <stdio.h>
#include <stdlib.h>

/* Do not define BIG_ENDIAN for PC/AT */
#ifdef sony_news
#define BIG_ENDIAN
#endif

void put2Bytes(short data, FILE* fp) {
	char* bytep = (char*)&data;

#ifdef BIG_ENDIAN
	putc(bytep[1], fp);
	putc(bytep[0], fp);
#else
	putc(bytep[0], fp);
	putc(bytep[1], fp);
#endif
	fflush(fp);
}

void put4Bytes(long data, FILE* fp) {
	char* bytep = (char*)&data;

#ifdef BIG_ENDIAN
	putc(bytep[3], fp);
	putc(bytep[2], fp);
	putc(bytep[1], fp);
	putc(bytep[0], fp);
#else
	putc(bytep[0], fp);
	putc(bytep[1], fp);
	putc(bytep[2], fp);
	putc(bytep[3], fp);
#endif
	fflush(fp);
}

#define	USAGE	"%s [-o outfile] wave1.wvf [wave2.wvf ...]\n"

static char DEFAULT_NAME[] = "wave.dat";

#define BUFSIZE 128

main(int argc, char** argv) 
{
	int	i = 1;			/* argument index   */
	char*	outfile = 0;		/* output file name */
	long	nWaves = 0;		/* number of waves  */
	FILE	*rfp, *wfp;		/* read/write file pointers */
	
	if ((argc > 1) && argv[1][0] == '-') {	/* check option */
		switch (argv[1][1]) {
		case ('o'):
		case ('O'):
			outfile = argv[2];
			i += 2;
			break;
		default:
			i = argc;	/* bad option */
		}
	}
	
	if (i >= argc) {		/* no input file */
		fprintf(stderr, USAGE, argv[0]);
		exit(0);
	}
	
	if (outfile == 0)		/* set default output file name */
		outfile = DEFAULT_NAME;

	if (!(wfp = fopen(outfile, "wb"))) {
		fprintf(stderr, "Cannot open %s\n", argv[1]);
		exit(1);
	}

	put4Bytes(0, wfp);		/* fill number of waves later */

	for (; i < argc; i++) {
		char	buf[BUFSIZE];
		int	nSamples = 0;		/* number of samples */
		
		if (!(rfp = fopen(argv[i], "r"))) {
			fprintf(stderr, "Cannot open %s\n", argv[1]);
			exit(1);
		}
		
		while (fgets(buf, BUFSIZE, rfp) != NULL) {

			if ((buf[0] == '#') || (buf[0] == '\n'))	/* skip comment & null lines */
				continue;
 
			if (sscanf(buf, "DATASIZE= %d", &nSamples) == 1) {
				int	j;
				
				put4Bytes((long)nSamples, wfp);		/* write number of samples */

				for (j = 0; j < nSamples; j++) {
					int sample;

					if (fgets(buf, BUFSIZE, rfp) == NULL) {
						fprintf(stderr, "Read Error (%s)\n", argv[i]);
						exit(1);
					}
					
					if ((buf[0] == '#') || (buf[0] == '\n'))	/* skip comment & null lines */
						continue;
					
					sample = atoi(buf);
					put4Bytes((long)sample, wfp);	/* write each sample */
				}
				++nWaves;
			}
		}
		fclose(rfp);
	}
	fseek(wfp, 0L, 0);		/* rewind */
	put4Bytes(nWaves, wfp);		/* write number of waves */

	fclose(wfp);
	return 0;
}
